/*
 *   Agri-monitor by Sigfox
 *   
 *   Jumper settings:
 *     J201: Open
 *     J202: Open
 *     J203: Close
 *     J204: 3.3V-side close
 *   Power supply:
 *     GND: Battery(AAA*2) -
 *     5V&3.3V:  Battery(AAA*2) +
 *   Grove connector:
 *     CONN#1: GY-49(max44009)
 *     CONN#2: Open
 */


#include <Wire.h>
#include <SoftwareSerial.h>
#include <SIGFOX.h>       // Use a partially modified one
#include <max44009.h>
#include <Seeed_BME280.h> // Use a partially modified one
#include <Narcoleptic.h>

// Constants
const uint32_t upload_interval = 60UL * 60UL * 1000UL;    // about 1 hour
const int ledPin = 9;
const bool echo = false; // Wisol library debug message off

// Globals
uint32_t last = 0;
const Country country = COUNTRY_JP;  //  Set this to your country to configure the SIGFOX transmission frequencies.
UnaShieldV2S transceiver(country, false, "NOTUSE", echo);  //  Uncomment this for UnaBiz UnaShield V2S Dev Kit
MAX44009 max44009(LOW);
BME280 bme280;

// resetMyself() - Reset myself 
void(* resetMyself) (void) = 0;

// setup() - set up function
void setup() {
  // begin sigfox module
  if (! transceiver.begin()) {
    blinkLed(7, true);  // Sigfox module error
  }

  // initialize bme280 sensor
  if (! bme280.init()){  
    blinkLed(5, true);  // BME280 sensor error
  }
}

// loop() - loop body function
void loop() {
  uint32_t start = millis();
  // Get sensor infomarion and upload them
  String message;
  if (getInformation(message)) {
    transceiver.sendMessage(message);
  }
  else {
    blinkLed(3, false);
  }
  // Sleep mcu until next time
  uint32_t t = (last + upload_interval) - (millis() - start);
  last = millis();
  bme280.end();  // Sleep BME280
  Wire.end();    // Release i2c
  transceiver.sleep();  // Sleep Sigfox module
  prepareToSleep();     // Set MCU GPIOs for sleep
  Narcoleptic.delay(t); // Sleep MCU with WDT
  // Wake up, restart from reboot
  resetMyself();
}

// getInformation() -- Get sensor information and assemble the message(payload)
boolean getInformation(String &message) {
  // Get sensing information and battery voltage
  int temperature = bme280.getTemperature() * 10;
  int humidity = bme280.getHumidity() * 10;
  uint32_t illuminance = max44009.getIlluminance();
  float fVoltage;
  transceiver.getVoltage(fVoltage);
  int voltage = fVoltage * 1000;  // Round off decimal places [mV]
  // Make the message
  message = transceiver.toHex(temperature);
  message += transceiver.toHex(humidity);
  message += transceiver.toHex(illuminance);
  message += transceiver.toHex(voltage);
  message += transceiver.toHex("OK", 2);    

  return true;  // OK
}

// prepareToSleep() - Prepare to sleep()
void prepareToSleep(void) {
  // Set GPIO pins for sleep
  for (int i = 0; i < 14; i++) {
    switch (i) {
      case 2:       // INT1 from BME280 - not used
      case 3:       // INT2 from BME280 - not used
        pinMode(i, INPUT_PULLUP);
        break;
      case 4:       // TX(output)
        pinMode(i, INPUT);
        break;
      case 5:       // RX(input)
        pinMode(i, INPUT_PULLUP);
        break;
      case 7:       // Wake up (Active:LOW) -  not used
        pinMode(i, INPUT_PULLUP);
        break;
      case 6:       // Pull up line - not used
        pinMode(i, INPUT_PULLUP);
        break;
      default:
        pinMode(i, OUTPUT);
        break;
    }
  }
  // disable ADC
  ADCSRA &= ~(1 << ADEN);
  // disable BOD
  MCUCR |= (1 << BODSE)|(1 << BODS);
  MCUCR = (MCUCR & ~(1 << BODSE))|(1 << BODS);
}

// blinkLed() -- Blink LED(D9) on Sigfox shield
void blinkLed(int times, boolean stop) {
  while (times-- > 0) {
    digitalWrite(ledPin, HIGH);
    delay(300); 
    digitalWrite(ledPin, LOW);
    delay(300); 
  }  
  if (stop) {
    // Stop execution here
    while (1)
      ;
  }
}

// END OF agri_mon2.ino
