#!/usr/bin/env python
# -*- coding:utf-8 -*-

'''
  ファイル名 : mitsuboshi.py

  起動後カメラ画像を取得しメールに添付して送信、シャットダウン。
  起動時に指定 GPIO を HIGH に設定、シャットダウン時に LOW に設定。

  開発環境 : Raspberry Pi 3 Model B、Pi 3 Model B+、Pi ZERO W
             Python 3.5.3

  2018/07 : Ver.0.1 初版

  Copyright 2018 Moo Soft(Akiyosi Yokoyama) https://moosoft.jp

  -------------------------------------------------------------------------
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
  -------------------------------------------------------------------------
'''
import time
import subprocess
import RPi.GPIO as GPIO

from moo_cam import CamImage      # カメラ画像作成クラス
from moo_sendMail import SendMail # メール送信クラス

# ============================================== #
# クラス
# ============================================== #
class MitsuBoshi(object):

    # ========================================== #
    # コンストラクタ
    # _end   : メール送信終了時に HIGH に設定する GPIO ピン番号
    # _sdown : 外部から LOW に設定されるとシャットダウンを行う GPIO ピン番号
    #
    # ピン番号指定は P1 ヘッダ上のピン番号を用いる
    # ========================================== #
    def __init__(self, _end=37, _sdown=29):
        self._end   = _end
        self._sdown = _sdown
        self.init_gpio()     # GPIO 初期化

    # ========================================== #
    # GPIO 初期化
    # 戻り値 : 無し
    #
    # GPIO.setmode(GPIO.BOARD) P1 ヘッダ上のピン番号を用いる
    # ========================================== #
    def init_gpio(self):
        try:
            GPIO.setwarnings(False)                    # Warning 表示停止
            GPIO.setmode(GPIO.BOARD)                   # P1 ヘッダ上のピン番号を用いる
            GPIO.setup(channel=self._end,              # GPIO ピン番号
                       direction=GPIO.OUT,             # 出力
                       initial=GPIO.LOW)               # 初期値 LOW

            GPIO.setup(channel=self._sdown,            # シャットダウン Pin 設定
                       direction=GPIO.IN,              # 入力
                       pull_up_down=GPIO.PUD_UP)       # プルアップ
            GPIO.add_event_detect(gpio=self._sdown,    # ピン番号
                                  edge=GPIO.FALLING,   # 検知モード(立下り)
                                  callback=self.sdown, # コールバック関数設定
                                  bouncetime=100)      # チャタリング対策
        except Exception as e:
            print('init_gpio:{0}'.format(e))

    # ========================================== #
    # ピン状態変化時のコールバック関数
    # ch     : ピン番号
    # 戻り値 : 無し
    #
    # 5秒以上同じ状態ならリブート or シャットダウン
    # ========================================== #
    def sdown(self, ch):
        print('ch:{0}'.format(ch))
        sd = 'sudo shutdown -h now'
        mcount = 5                                  # カウンタ閾値
        try:
            count = 1                               # カウンタ初期値
            while GPIO.input(ch) == GPIO.LOW:       # LOW が続いていれば
                time.sleep(1.0)
                print('Count:{0}'.format(count))
                count += 1                          # カウントアップ
                if count > mcount:                  # 閾値を超えたら
                    print(sd)
                    GPIO.cleanup()
                    subprocess.call(sd, shell=True) # 実行
        except Exception as e:
            print('sdown:{0}'.format(e))

    # ========================================== #
    # self._end を HIGH に設定する(メール送信完了)
    # 戻り値 : 無し
    # ========================================== #
    def set_high(self):
        try:
            GPIO.output(self._end, GPIO.HIGH)
        except Exception as e:
            print('set_high:{0}'.format(e))

    # ============================================== #
    # インターネット接続確認
    # url    : 通信確認先サーバー情報(Google DNS:8.8.8.8)
    # 戻り値 : 接続 = True それ以外 = False
    #
    # nc -z:ポートスキャン -w 5:タイムアウト指定
    # ============================================== #
    def has_internet(self, url='8.8.8.8 53'):
        result = False
        try:
            netcmd = 'nc -z -w 5 {0}'.format(url) # コマンド文字列作成
            result = self.get_os_result(netcmd)   # コマンド実行
        except Exception as e:
            print('has_internet:{0}'.format(e))
        return result

    # ============================================== #
    # インターネット接続確認 接続していれば ntpdate で時刻同期
    # url    : 通信確認先サーバー情報(Google DNS:8.8.8.8)
    # 戻り値 : 接続 = True それ以外 = False
    #
    #  【必用】 sudo apt-get install ntpdate
    # sudo ntpdate ntp.nict.jp で the NTP socket is in use, exiting エラーが発生したら
    # sudo ntpdate -u ntp.nict.jp -> -u オプションを付ける
    # -u パケットを非特権モードで出力する。
    # ============================================== #
    def has_internetEx(self, url='8.8.8.8 53'):
        result = False
        try:
            result = self.has_internet(url)                             # コマンド実行
            if result:                                                  # インターネット接続していれば
                result = self.get_os_result('sudo ntpdate ntp.nict.jp') # ntpdate で時刻同期 ntp.jst.mfeed.ad.jp
        except Exception as e:
            print('has_internetEx:{0}'.format(e))
        return result

    # ============================================== #
    # コマンドを実行して戻り値を取得する
    # cmd    : 実行コマンド文字列
    # 戻り値 : 成功 = True それ以外 = False
    # ============================================== #
    def get_os_result(self, cmd):
        result = False
        try:
            subpon = subprocess.Popen([cmd], shell=True) # コマンド実行 必ず shell=True
            result = subpon.wait() == 0                  # 戻り値チェック
            cmd_str  = 'cmd    : {0}\n'
            cmd_str += 'result : {1}'
            print(cmd_str.format(cmd, result))
            print('-' * 20)
        except Exception as e:
            subpon.kill()
            print('get_os_result:{0}'.format(e))
        return result

# end class MitsuBoshi

# ============================================== #
# メイン
# ============================================== #
def main():
    # *** 環境に合わせて変更してください *** #
    gpath   = '/home/pi/prg/tmp/cam.jpg'      # カメラ画像保存パス
    mlfrom  = 'hoge@gmail.com'                # メール送信元
    server  = 'smtp.gmail.com'                # SMTP メールサーバー
    port    = 587                             # SMTP ポート
    acount  = 'hoge@gmail.com'                # メールアカウント
    passwd  = 'hoge_fuga'                     # メールパスワード
    mail_to = 'hoge@gmail.com;fuga@gmail.com' # メール送信先 複数指定は ';' で区切る
    subject = '件名テスト'                    # メール件名
    body    = '本文テスト'                    # メール本文
    # ************************************** #

    try:
        m3    = MitsuBoshi(_end=37, _sdown=29)           # クラスインスタンス
        isok  = False
        count = 0
        while True:
            isok = m3.has_internetEx()                   # インターネット接続チェック,時刻同期
            if isok: break
            count += 1
            if count > 5: break                          # 試行回数
            print(count)
            time.sleep(10)
        if isok:                                         # インターネット接続があれば
            while True:
                isok = CamImage().get_image(gpath=gpath) # カメラ画像作成
                if isok: break
                count += 1
                if count > 5: break                      # 試行回数
                print(count)
                time.sleep(10)
            sm = SendMail(debug=True)                    # メール送信
            sm.smtp_mlfrom  = mlfrom
            sm.smtp_server  = server
            sm.smtp_port    = port
            sm.smtp_acount  = acount
            sm.smtp_passwd  = passwd
            sm.smtp_subject = subject
            sm.smtp_body    = body
            sm.smtp_afile   = gpath
            isok = sm.mail_to(mail_to)
            print('mail_to : {0}'.format(isok))
        m3.set_high()                                    # メール送信完了 GPIO 設定
        while True: time.sleep(600)
    except KeyboardInterrupt:
        GPIO.cleanup()
    except Exception as e:
        print('main:{0}'.format(e))

# ============================================== #
if __name__=='__main__':
    main()
