#!/usr/bin/env python
# -*- coding: UTF-8 -*-

'''
  ファイル名 : moo_sendMail.py

  メール送信クラス

  開発環境 : Raspberry Pi3 model B、Pi3 model B+、ZERO W
             Python 3.5.3

  2018/06 : Ver.0.1 初版

  Copyright 2018 Moo Soft(Akiyosi Yokoyama) https://moosoft.jp

  -------------------------------------------------------------------------
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
  -------------------------------------------------------------------------
'''

import os.path
import time
from datetime import datetime

import smtplib
from email.mime.text import MIMEText
from email.header import Header
from email.utils import formatdate

from email.mime.multipart import MIMEMultipart
from email.mime.image import MIMEImage
import mimetypes

from email import encoders
from email.message import Message
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase

# ============================================== #
# メール送信クラス
# ============================================== #
class SendMail(object):

    # ========================================== #
    # コンストラクタ
    # mlfrom  : 送信元メールアドレス
    # server  : SMTP サーバーアドレス
    # port    : SMTP ポート(初期値 : 587)
    # acount  : メールアカウント
    # passwd  : メールパスワード
    # subject : メール件名
    # body    : 本文
    # afile   : 添付ファイル
    # debug   : デバッグモードフラグ
    # ========================================== #
    def __init__(self, mlfrom='', server='', port=587, acount='', passwd='', subject='', body='', afile='', debug=False):
        self.debug   = debug
        self.mlfrom  = mlfrom
        self.server  = server
        self.port    = port
        self.acount  = acount
        self.passwd  = passwd
        self.subject = subject
        self.body    = body
        self.afile   = afile

    # ========================================== #
    # smtp_mlfrom getter
    # 戻り値 : 送信元メールアドレス
    # ========================================== #
    @property
    def smtp_mlfrom(self):
        return self.mlfrom

    # ========================================== #
    # smtp_mlfrom setter
    # val    : 送信元メールアドレス
    # 戻り値 : 無し
    # ========================================== #
    @smtp_mlfrom.setter
    def smtp_mlfrom(self, val):
        self.mlfrom = val

    # ========================================== #
    # smtp_server getter
    # 戻り値 : メール送信サーバーアドレス
    # ========================================== #
    @property
    def smtp_server(self):
        return self.server

    # ========================================== #
    # smtp_server setter
    # val    : メール送信サーバーアドレス
    # 戻り値 : 無し
    # ========================================== #
    @smtp_server.setter
    def smtp_server(self, val):
        self.server = val

    # ========================================== #
    # smtp_port getter
    # 戻り値 : メール送信ポート
    # ========================================== #
    @property
    def smtp_port(self):
        return self.port

    # ========================================== #
    # smtp_port setter
    # val    : メール送信ポート
    # 戻り値 : 無し
    # ========================================== #
    @smtp_port.setter
    def smtp_port(self, val):
        self.port = val

    # ========================================== #
    # acount getter
    # 戻り値 : メールアカウント
    # ========================================== #
    @property
    def smtp_acount(self):
        return self.acount

    # ========================================== #
    # acount setter
    # val    : メールアカウント
    # 戻り値 : 無し
    # ========================================== #
    @smtp_acount.setter
    def smtp_acount(self, val):
        self.acount = val

    # ========================================== #
    # passwd getter
    # 戻り値 : メールパスワード
    # ========================================== #
    @property
    def smtp_passwd(self):
        return self.passwd

    # ========================================== #
    # passwd setter
    # val    : メールパスワード
    # 戻り値 : 無し
    # ========================================== #
    @smtp_passwd.setter
    def smtp_passwd(self, val):
        self.passwd = val

    # ========================================== #
    # smtp_subject getter
    # 戻り値 : 件名
    # ========================================== #
    @property
    def smtp_subject(self):
        return self.subject

    # ========================================== #
    # smtp_subject setter
    # val    : 件名
    # 戻り値 : 無し
    # ========================================== #
    @smtp_subject.setter
    def smtp_subject(self, val):
        self.subject = val

    # ========================================== #
    # smtp_body getter
    # 戻り値 : 本文
    # ========================================== #
    @property
    def smtp_body(self):
        return self.body

    # ========================================== #
    # smtp_body setter
    # val    : 本文
    # 戻り値 : 無し
    # ========================================== #
    @smtp_body.setter
    def smtp_body(self, val):
        self.body = val

    # ========================================== #
    # smtp_afile getter
    # 戻り値 : 添付ファイルパス
    # ========================================== #
    @property
    def smtp_afile(self):
        return self.afile

    # ========================================== #
    # smtp_afile setter
    # val    : 添付ファイルパス
    # 戻り値 : 無し
    # ========================================== #
    @smtp_afile.setter
    def smtp_afile(self, val):
        self.afile = val

    # ========================================== #
    # パラメータ表示
    # 戻り値 : 無し
    # ========================================== #
    def print_params(self):
        try:
            print('mlfrom  : {0}'.format(self.mlfrom))
            print('server  : {0}'.format(self.server))
            print('port    : {0}'.format(self.port))
            print('acount  : {0}'.format(self.acount))
            print('passwd  : {0}'.format(self.passwd))
            print('subject : {0}'.format(self.subject))
            print('body    : {0}'.format(self.body))
            print('afile   : {0}'.format(self.afile))
        except Exception as e:
            print('print_params:{0}'.format(e))

    # ========================================== #
    # メール送信
    # mlto   : 送信先メールアドレス
    # 戻り値 : 無し
    # ========================================== #
    def mail_to(self, mlto):
        result = False
        try:
            mlto = mlto.strip()
            if mlto == '': return result

            body    = self.body
            subject = self.subject
            mm = MIMEMultipart()                                         # マルチパート
            mt = MIMEText(body.encode('utf-8'), 'plain', 'utf-8')        # プレンテキスト
            try:
                if os.path.isfile(self.afile):                           # 添付ファイルチェック
                    ctype, encoding = mimetypes.guess_type(self.afile)   # 拡張子から MIME タイプを推測する
                    if ctype is None or encoding is not None:
                        ctype = 'application/octet-stream'
                    maintype, subtype = ctype.split('/', 1)              # '/' で1回分割
                    if maintype == 'text':
                        with open(self.afile, encoding='utf-8') as f:
                            fp = f.read()
                        mj = MIMEText(fp, _subtype=subtype, _charset='utf-8')
                    elif maintype == 'image':
                        fp = open(self.afile, 'rb')
                        mj = MIMEImage(fp.read(), _subtype=subtype)
                        fp.close()
                    elif maintype == 'audio':
                        fp = open(self.afile, 'rb')
                        mj = MIMEAudio(fp.read(), _subtype=subtype)
                        fp.close()
                    else:
                        fp = open(self.afile, 'rb')
                        mj = MIMEBase(maintype, subtype)
                        mj.set_payload(fp.read())
                        fp.close()
                        encoders.encode_base64(mj)                       # Base64 エンコード
                    mj.add_header('Content-Disposition',                 # ファイル名設定
                                  'attachment',
                                  filename=os.path.basename(self.afile))
                    mm.attach(mj)                                        # 添付ファイル追加
                    mm.attach(mt)                                        # 本文追加
                else:
                    mm = mt
            except Exception as e:
                print('afile:{0}'.format(e))
                mm = mt
            mm['Subject'] = Header(subject, 'utf-8')
            mm['From']    = self.mlfrom
            mm['To']      = mlto
            mm['Date']    = formatdate()
            if self.debug: self.print_params()               # デバッグ
            ii = 0                                           # リトライカウンター
            while True:
                try:                                         # メール送信
                    s = smtplib.SMTP(self.server, self.port)
                    s.ehlo()                                 # EHLO を利用し、ESMTP サーバーに身元を明かす
                    s.starttls()                             # TLS(Transport Layer Security)モード
                    s.ehlo()                                 # 再度 EHLO を利用し、ESMTP サーバーに身元を明かす
                    s.login(self.acount, self.passwd)        # アカウント、パスワード
                    try:
                        s.sendmail(self.mlfrom,
                                   mlto.split(';'),          # 複数の宛先は ';' で区切る
                                   mm.as_string())
                        result = True
                    finally:
                        s.quit()                             # SMTP セッションを終了し、コネクションを閉じる
                    break
                except Exception as e:
                    print('tls_sendthread Count:{0}:{1}'.format(ii, e))
                    ii += 1
                    if ii > 2: break
                    time.sleep(10)
        except Exception as e:
            print('mail_to():{0}'.format(e))
        return result

# end class SendMail

# ============================================== #
# メイン
# ============================================== #
def main():
    try:
        sm = SendMail(debug=True)
        sm.smtp_mlfrom  = 'hoge@gmail.com'
        sm.smtp_server  = 'smtp.gmail.com'
        sm.smtp_port    = 587
        sm.smtp_acount  = 'hoge@gmail.com'
        sm.smtp_passwd  = 'hoge_fuga'
        sm.smtp_subject = '件名テスト'
        sm.smtp_body    = '本文テスト'
        #sm.smtp_afile  = '/home/pi/prg/a.zip'
        result = sm.mail_to('hoge@gmail.com;fuga@gmail.com')
        print('mail_to : {0}'.format(result))
    except Exception as e:
        print('main:{0}'.format(e))

# ============================================== #
if __name__=='__main__':
    main()
