#include <esp_deep_sleep.h>
#include <WiFi.h>
#include <HTTPClient.h>

#define PULSE_PIN 35 //水流量センサからの入力（GPIO35）
#define V_PIN   34   //電圧モニタ入力（GPIO34）

#define REC_SIZE 60
#define STOP_CNT 5

#define PULSE_PER_LITER 450

#define SSID "Wi-FiアクセスポイントのSSID"
#define PASS "Wi-FIアクセスポイントのパスワード"

volatile uint32_t pulse_counter;

portMUX_TYPE mux = portMUX_INITIALIZER_UNLOCKED;
#define DI() portENTER_CRITICAL_ISR(&mux);
#define EI() portEXIT_CRITICAL_ISR(&mux);

/*
 * カウンタをインクリメント（割り込み処理）・・・①
 */
void IRAM_ATTR handler() {
  DI();
  pulse_counter++;
  EI();  
}

/*
 * １秒間パルスをカウントする
 */
uint16_t count_pulse() {
  uint16_t pulse;
  DI();
  pulse_counter = 0;
  EI();
  delay(1000);
  DI();
  pulse = pulse_counter;
  EI();
  return pulse;
}

/*
 * センサ値をサーバへ送信する
 */
void push_server(float waterflow, float voltage) {
  char s[36];
  sprintf(s, "{\"flow\":\"%.2f\",\"voltage\":\"%.2f\"}", waterflow, voltage);

  HTTPClient http;
　　//Google Functionsで作成したエンドポイントに書き換える
  http.begin("https://asia-northeast1-????.cloudfunctions.net/post"); 
  http.addHeader("Content-Type", "application/json");

  int response = http.POST(s);

  if (response > 0) {
    Serial.println(http.getString());
  } else {
    Serial.println(response);
  }

  http.end();  
}

void setup() {
  Serial.begin(115200);
  delay(10);
  Serial.println("wakeup ESP32");
  
　　//パルス入力の割り込み設定
  pinMode(PULSE_PIN, INPUT);
  attachInterrupt(digitalPinToInterrupt(PULSE_PIN), handler, RISING);

　　//60秒間パルスをカウントする
  uint8_t zero_count = 0;
  uint16_t total_pulse = 0;
  uint16_t pulse;
  for (int i = 0; i < REC_SIZE; i++) {
    pulse = count_pulse();
    total_pulse += pulse;
    Serial.print(pulse); Serial.print("/"); Serial.println(total_pulse);
    if (pulse == 0) {
      zero_count++;
      if (zero_count > STOP_CNT) break;
    } else {
      zero_count = 0;
    }
  }

  float waterflow = (float)total_pulse / PULSE_PER_LITER;
  float voltage = ((float)analogRead(V_PIN) / 4096) * 3.3 * 1.5;
  Serial.print("P:"); Serial.println(waterflow);
  Serial.print("V:"); Serial.println(voltage);

  if (waterflow > 0 || esp_sleep_get_wakeup_cause() == ESP_SLEEP_WAKEUP_TIMER) {
    WiFi.begin(SSID, PASS); //Wi-Fi接続開始

    Serial.println(); Serial.println();
    Serial.print("Wait for WiFi...");
    while (WiFi.status() == WL_NO_SHIELD) {
      Serial.print(".");
      delay(500);
    }
    switch(WiFi.status()) {
    case WL_NO_SSID_AVAIL:
      Serial.println("ERROR:No SSID available");
      break;
    case WL_CONNECTED:
      Serial.println();
      Serial.print("WiFi connected/IP address:");
      Serial.println(WiFi.localIP());
      push_server(waterflow, voltage);//データ送信
      break;
    default:
      Serial.println("ERROR:WiFi not connected");
      break;
    }
  }


  /*
   * Deep Sleepモードへ移行
   */
  Serial.println("deep sleep");
  esp_deep_sleep_pd_config(ESP_PD_DOMAIN_RTC_PERIPH, ESP_PD_OPTION_AUTO);
  
  /*GPIO35の変化時にWakeUp・・・②
    現在のGPIO35のレベルを確認し，その反対をWakeUp要因として設定する*/
  int state = digitalRead(PULSE_PIN);
  esp_deep_sleep_enable_ext0_wakeup(GPIO_NUM_35, !state);

  /* 1時間後にWakeUp・・・③ */
  uint64_t sleep_time = 60 * 60 * 1000000ULL;
  esp_deep_sleep_enable_timer_wakeup(sleep_time);
  
  esp_deep_sleep_start(); //スリープ開始
  
}

void loop() {
  
}
